@@ -171,6 +171,17 @@ A string that will be prepended to topics on sent messages.
 This is useful to migrate from fedmsg, but should not be used otherwise.
 The default is an empty string.
 
+.. _conf-publish-priority:
+
+publish_priority
+----------------
+A number that will be set as the priority for the messages. The range of
+possible priorities depends on the ``x-max-priority`` argument of the
+destination queue, as described in `RabbitMQ's priority documentation`_.
+The default is ``None``, which RabbitMQ will interpret as zero.
+
+.. _RabbitMQ's priority documentation: https://www.rabbitmq.com/priority.html
+
 .. _sub-config:
 
 Consumer Options
@@ -285,7 +296,12 @@ import logging.config
 import os
 
 import pkg_resources
-import toml
+
+
+try:
+    import tomllib
+except ImportError:
+    import tomli as tomllib
 
 from . import exceptions
 
@@ -316,6 +332,7 @@ DEFAULTS = dict(
     },
     publish_exchange="amq.topic",
     topic_prefix="",
+    publish_priority=None,
     passive_declares=False,
     exchanges={
         "amq.topic": {
@@ -521,15 +538,13 @@ class LazyConfig(dict):
 
         if os.path.exists(config_path):
             _log.info(f"Loading configuration from {config_path}")
-            with open(config_path) as fd:
+            with open(config_path, "rb") as fd:
                 try:
-                    file_config = toml.load(fd)
+                    file_config = tomllib.load(fd)
                     for key in file_config:
                         config[key.lower()] = file_config[key]
-                except toml.TomlDecodeError as e:
-                    msg = "Failed to parse {}: error at line {}, column {}: {}".format(
-                        config_path, e.lineno, e.colno, e.msg
-                    )
+                except tomllib.TOMLDecodeError as e:
+                    msg = f"Failed to parse {config_path}: {e}"
                     raise exceptions.ConfigurationException(msg)
         else:
             _log.info(f"The configuration file, {config_path}, does not exist.")
