@@ -213,6 +213,16 @@ def get_message(routing_key, properties, body):
     except jsonschema.exceptions.ValidationError as e:
         _log.error("Message validation of %r failed: %r", message, e)
         raise ValidationError(e)
+
+    if MessageClass.deprecated:
+        _log.warning(
+            "A message with a deprecated schema (%s.%s) has been received on topic %r. "
+            "You should check the emitting application's documentation to upgrade to "
+            "the newer schema version.",
+            MessageClass.__module__,
+            MessageClass.__name__,
+            message.topic,
+        )
     return message
 
 
@@ -280,6 +290,21 @@ class Message:
         queue (str): The name of the queue this message arrived through. This
             attribute is set automatically by the library and users should never
             set it themselves.
+        deprecated (bool): Whether this message schema has been deprecated by a more
+            recent version. Emits a warning when a message of this class is received,
+            to let consumers know that they should plan to upgrade. Defaults to
+            ``False``.
+        priority (int): The priority for the message, if the destination queue
+            supports it. Defaults to zero (lowest priority).
+
+            This value is taken into account in queues that have the ``x-max-priority``
+            argument set. Most queues in Fedora don't support priorities, in which case
+            the value will be ignored.
+
+            Larger numbers indicate higher priority, you can read more about it in
+            `RabbitMQ's documentation on priority`_.
+
+            .. _RabbitMQ's documentation on priority:  https://www.rabbitmq.com/priority.html
     """
 
     severity = INFO
@@ -302,6 +327,7 @@ class Message:
         "description": "Schema for message body",
         "type": "object",
     }
+    deprecated = False
 
     def __init__(
         self, body=None, headers=None, topic=None, properties=None, severity=None
@@ -333,6 +359,7 @@ class Message:
             delivery_mode=2,
             headers=headers,
             message_id=message_id,
+            priority=config.conf["publish_priority"],
         )
 
     def _filter_headers(self):
@@ -379,6 +406,14 @@ class Message:
     def id(self, value):
         self._properties.message_id = value
 
+    @property
+    def priority(self):
+        return self._properties.priority or 0
+
+    @priority.setter
+    def priority(self, value):
+        self._properties.priority = value
+
     @property
     def _encoded_routing_key(self):
         """The encoded routing key used to publish the message on the broker."""
@@ -658,6 +693,10 @@ SERIALIZED_MESSAGE_SCHEMA = {
             "type": ["string", "null"],
             "description": "The queue the message arrived on, if any.",
         },
+        "priority": {
+            "type": ["integer", "null"],
+            "description": "The priority that the message has been sent with.",
+        },
     },
     "required": ["topic", "headers", "id", "body"],
 }
@@ -695,6 +734,7 @@ def dumps(messages):
             "id": message.id,
             "body": message.body,
             "queue": message.queue,
+            "priority": message.priority,
         }
         serialized_messages.append(json.dumps(m, ensure_ascii=False, sort_keys=True))
 
@@ -736,6 +776,7 @@ def loads(serialized_messages):
         )
         message.queue = message_dict["queue"] if "queue" in message_dict else None
         message.id = message_dict["id"]
+        message.priority = message_dict.get("priority")
         messages.append(message)
 
     return messages
