@@ -4,18 +4,18 @@ import socket
 import time
 import uuid
 from collections import defaultdict
+from unittest import mock
+from urllib.parse import quote
 
-import mock
-import pika
 import pytest
 import pytest_twisted
-import six
 import treq
-from fedora_messaging import api, config, exceptions, message
-from fedora_messaging.twisted import service
-from fedora_messaging.twisted.protocol import _add_timeout
 from twisted.internet import defer, reactor, task, threads
 
+from fedora_messaging import api, config, exceptions, message
+from fedora_messaging.twisted.consumer import _add_timeout
+
+
 HTTP_API = "http://localhost:15672/api/"
 HTTP_AUTH = ("guest", "guest")
 
@@ -42,14 +42,14 @@ def admin_user():
     """
     # Create a user with no permissions
     username = str(uuid.uuid4())
-    url = "{base}users/{user}".format(base=HTTP_API, user=username)
+    url = f"{HTTP_API}users/{username}"
     body = {"username": username, "password": "guest", "tags": "administrator"}
     deferred_resp = treq.put(url, json=body, auth=HTTP_AUTH, timeout=3)
 
     @pytest_twisted.inlineCallbacks
     def cp(resp):
         assert resp.code == 201
-        url = "{base}permissions/%2F/{user}".format(base=HTTP_API, user=username)
+        url = f"{HTTP_API}permissions/%2F/{username}"
         body = {"configure": ".*", "write": ".*", "read": ".*"}
         resp = yield treq.put(url, json=body, auth=HTTP_AUTH, timeout=3)
         assert resp.code == 201
@@ -80,7 +80,7 @@ def get_queue(queue_dict, delay=10):
     name = queues[0]
     # Assert both messages are delivered, no messages are un-acked, and only one
     # message got a positive acknowledgment.
-    url = "{base}queues/%2F/{queue}".format(base=HTTP_API, queue=name)
+    url = f"{HTTP_API}queues/%2F/{name}"
     server_queue = yield task.deferLater(
         reactor, delay, treq.get, url, auth=HTTP_AUTH, timeout=3
     )
@@ -111,14 +111,14 @@ def test_twisted_consume_halt_consumer(queue_and_binding):
     """
     queues, bindings = queue_and_binding
     msg = message.Message(
-        topic=u"nice.message",
-        headers={u"niceness": u"very"},
-        body={u"encouragement": u"You're doing great!"},
+        topic="nice.message",
+        headers={"niceness": "very"},
+        body={"encouragement": "You're doing great!"},
     )
     expected_headers = {
-        u"fedora_messaging_severity": 20,
-        u"fedora_messaging_schema": u"base.message",
-        u"niceness": u"very",
+        "fedora_messaging_severity": 20,
+        "fedora_messaging_schema": "base.message",
+        "niceness": "very",
     }
     messages_received = []
 
@@ -144,8 +144,8 @@ def test_twisted_consume_halt_consumer(queue_and_binding):
         assert len(messages_received) == 3
         assert e.exit_code == 0
         for m in messages_received:
-            assert u"nice.message" == m.topic
-            assert {u"encouragement": u"You're doing great!"} == m.body
+            assert "nice.message" == m.topic
+            assert {"encouragement": "You're doing great!"} == m.body
             assert "sent-at" in m._headers
             del m._headers["sent-at"]
             assert expected_headers == m._headers
@@ -230,9 +230,9 @@ def test_twisted_consume_halt_consumer_requeue(queue_and_binding):
     """Assert raising HaltConsumer with requeue=True re-queues the message."""
     queues, bindings = queue_and_binding
     msg = message.Message(
-        topic=u"nice.message",
-        headers={u"niceness": u"very"},
-        body={u"encouragement": u"You're doing great!"},
+        topic="nice.message",
+        headers={"niceness": "very"},
+        body={"encouragement": "You're doing great!"},
     )
 
     def callback(message):
@@ -265,9 +265,9 @@ def test_twisted_consume_drop_message(queue_and_binding):
     """Assert raising Drop causes the message to be dropped, but processing continues."""
     queues, bindings = queue_and_binding
     msg = message.Message(
-        topic=u"nice.message",
-        headers={u"niceness": u"very"},
-        body={u"encouragement": u"You're doing great!"},
+        topic="nice.message",
+        headers={"niceness": "very"},
+        body={"encouragement": "You're doing great!"},
     )
     dropped_messages = []
 
@@ -305,9 +305,9 @@ def test_twisted_consume_nack_message(queue_and_binding):
     """Assert raising Nack causes the message to be replaced in the queue."""
     queues, bindings = queue_and_binding
     msg = message.Message(
-        topic=u"nice.message",
-        headers={u"niceness": u"very"},
-        body={u"encouragement": u"You're doing great!"},
+        topic="nice.message",
+        headers={"niceness": "very"},
+        body={"encouragement": "You're doing great!"},
     )
     nacked_messages = []
 
@@ -346,9 +346,9 @@ def test_twisted_consume_general_exception(queue_and_binding):
     """
     queues, bindings = queue_and_binding
     msg = message.Message(
-        topic=u"nice.message",
-        headers={u"niceness": u"very"},
-        body={u"encouragement": u"You're doing great!"},
+        topic="nice.message",
+        headers={"niceness": "very"},
+        body={"encouragement": "You're doing great!"},
     )
 
     def callback(message):
@@ -388,9 +388,9 @@ def test_twisted_consume_connection_reset(queue_and_binding):
     """
     queues, bindings = queue_and_binding
     msg = message.Message(
-        topic=u"nice.message",
-        headers={u"niceness": u"very"},
-        body={u"encouragement": u"You're doing great!"},
+        topic="nice.message",
+        headers={"niceness": "very"},
+        body={"encouragement": "You're doing great!"},
     )
     messages_received = []
     two_received = defer.Deferred()  # Fired by the callback on 2 messages
@@ -427,7 +427,7 @@ def test_twisted_consume_connection_reset(queue_and_binding):
             and c["client_properties"]["app"] == "test_twisted_consume_connection_reset"
         ]
         if this_conn:
-            cname = six.moves.urllib.parse.quote(this_conn[0]["name"])
+            cname = quote(this_conn[0]["name"])
             yield treq.delete(
                 HTTP_API + "connections/" + cname, auth=HTTP_AUTH, timeout=3
             )
@@ -459,7 +459,7 @@ def test_twisted_consume_serverside_cancel(queue_and_binding):
     consumers = yield api.twisted_consume(lambda x: x, bindings, queues)
 
     # Delete the queue and assert the consumer errbacks
-    url = "{base}queues/%2F/{queue}".format(base=HTTP_API, queue=list(queues.keys())[0])
+    url = f"{HTTP_API}queues/%2F/{list(queues.keys())[0]}"
     yield treq.delete(url, auth=HTTP_AUTH, timeout=3)
 
     _add_timeout(consumers[0].result, 10)
@@ -475,13 +475,13 @@ def test_twisted_consume_serverside_cancel(queue_and_binding):
 @pytest_twisted.inlineCallbacks
 def test_no_vhost_permissions(admin_user, queue_and_binding):
     """Assert a hint is given if the user doesn't have any access to the vhost"""
-    url = "{base}permissions/%2F/{user}".format(base=HTTP_API, user=admin_user)
+    url = f"{HTTP_API}permissions/%2F/{admin_user}"
     resp = yield treq.delete(url, auth=HTTP_AUTH, timeout=3)
     assert resp.code == 204
 
     queues, _ = queue_and_binding
 
-    amqp_url = "amqp://{user}:guest@localhost:5672/%2F".format(user=admin_user)
+    amqp_url = f"amqp://{admin_user}:guest@localhost:5672/%2F"
     with mock.patch.dict(config.conf, {"amqp_url": amqp_url}):
         try:
             yield api.twisted_consume(lambda x: x, [], queues)
@@ -503,12 +503,12 @@ def test_no_read_permissions_queue_read_failure_pika1(admin_user, queue_and_bind
     what errors back.
     """
     queues, bindings = queue_and_binding
-    url = "{base}permissions/%2F/{user}".format(base=HTTP_API, user=admin_user)
+    url = f"{HTTP_API}permissions/%2F/{admin_user}"
     body = {"configure": ".*", "write": ".*", "read": ""}
     resp = yield treq.put(url, json=body, auth=HTTP_AUTH, timeout=3)
     assert resp.code == 204
 
-    amqp_url = "amqp://{user}:guest@localhost:5672/%2F".format(user=admin_user)
+    amqp_url = f"amqp://{admin_user}:guest@localhost:5672/%2F"
     with mock.patch.dict(config.conf, {"amqp_url": amqp_url}):
         try:
             consumers = api.twisted_consume(lambda x: x, [], queues)
@@ -528,12 +528,12 @@ def test_no_read_permissions_queue_read_failure_pika1(admin_user, queue_and_bind
 def test_no_read_permissions_bind_failure(admin_user, queue_and_binding):
     """Assert the call to twisted_consume errbacks on read permissions errors on binding."""
     queues, bindings = queue_and_binding
-    url = "{base}permissions/%2F/{user}".format(base=HTTP_API, user=admin_user)
+    url = f"{HTTP_API}permissions/%2F/{admin_user}"
     body = {"configure": ".*", "write": ".*", "read": ""}
     resp = yield treq.put(url, json=body, auth=HTTP_AUTH, timeout=3)
     assert resp.code == 204
 
-    amqp_url = "amqp://{user}:guest@localhost:5672/%2F".format(user=admin_user)
+    amqp_url = f"amqp://{admin_user}:guest@localhost:5672/%2F"
     try:
         with mock.patch.dict(config.conf, {"amqp_url": amqp_url}):
             yield api.twisted_consume(lambda x: x, bindings, queues)
@@ -550,12 +550,12 @@ def test_no_read_permissions_bind_failure(admin_user, queue_and_binding):
 def test_no_write_permissions(admin_user, queue_and_binding):
     """Assert the call to twisted_consume errbacks on write permissions errors."""
     queues, bindings = queue_and_binding
-    url = "{base}permissions/%2F/{user}".format(base=HTTP_API, user=admin_user)
+    url = f"{HTTP_API}permissions/%2F/{admin_user}"
     body = {"configure": ".*", "write": "", "read": ".*"}
     resp = yield treq.put(url, json=body, auth=HTTP_AUTH, timeout=3)
     assert resp.code == 204
 
-    amqp_url = "amqp://{user}:guest@localhost:5672/%2F".format(user=admin_user)
+    amqp_url = f"amqp://{admin_user}:guest@localhost:5672/%2F"
     try:
         with mock.patch.dict(config.conf, {"amqp_url": amqp_url}):
             yield api.twisted_consume(lambda x: x, bindings, queues)
@@ -673,12 +673,12 @@ def test_protocol_publish_connection_error(queue_and_binding):
 @pytest_twisted.inlineCallbacks
 def test_protocol_publish_forbidden(admin_user):
     """Assert individual protocols raise exceptions when the topic is not allowed."""
-    url = "{base}topic-permissions/%2F/{user}".format(base=HTTP_API, user=admin_user)
+    url = f"{HTTP_API}topic-permissions/%2F/{admin_user}"
     body = {"exchange": "amq.topic", "write": "^allowed$", "read": ".*"}
     resp = yield treq.put(url, json=body, auth=HTTP_AUTH, timeout=3)
     assert resp.code == 201
 
-    amqp_url = "amqp://{user}:guest@localhost:5672/%2F".format(user=admin_user)
+    amqp_url = f"amqp://{admin_user}:guest@localhost:5672/%2F"
     with mock.patch.dict(config.conf, {"amqp_url": amqp_url}):
         try:
             api._init_twisted_service()
@@ -699,6 +699,34 @@ def test_protocol_publish_forbidden(admin_user):
         yield protocol.publish(message.Message(topic="allowed"), "amq.topic")
 
 
+@pytest_twisted.inlineCallbacks
+def test_protocol_publish_forbidden_in_vhost(admin_user):
+    """Assert individual protocols raise a forbidden exception immediately when
+    the user is not allowed to publish against the virtual host."""
+    url = f"{HTTP_API}permissions/%2F/{admin_user}"
+    body = {"configure": ".*", "write": "", "read": ".*"}
+    resp = yield treq.put(url, json=body, auth=HTTP_AUTH, timeout=3)
+    assert resp.code == 204
+
+    amqp_url = f"amqp://{admin_user}:guest@localhost:5672/%2F"
+    with mock.patch.dict(config.conf, {"amqp_url": amqp_url}):
+        try:
+            api._init_twisted_service()
+            protocol = yield api._twisted_service._service.factory.when_connected()
+            d = protocol.publish(message.Message(topic="not-allowed"), "amq.topic")
+            d.addTimeout(5, reactor)
+            yield d
+            pytest.fail("Publish failed to raise an exception")
+        except (defer.TimeoutError, defer.CancelledError):
+            pytest.fail("Publishing hit the timeout, probably stuck in a retry loop")
+        except exceptions.PublishForbidden as e:
+            assert e.reason.args[0] == 403
+            assert e.reason.args[1] == (
+                "ACCESS_REFUSED - access to exchange 'amq.topic'"
+                " in vhost '/' refused for user '{}'".format(admin_user)
+            )
+
+
 @pytest_twisted.inlineCallbacks
 def test_pub_sub_default_settings(queue_and_binding):
     """
@@ -724,9 +752,9 @@ def test_pub_sub_default_settings(queue_and_binding):
     def delayed_publish():
         """Give the consumer time to setup."""
         msg = message.Message(
-            topic=u"nice.message",
-            headers={u"niceness": u"very"},
-            body={u"encouragement": u"You're doing great!"},
+            topic="nice.message",
+            headers={"niceness": "very"},
+            body={"encouragement": "You're doing great!"},
         )
         for _ in range(0, 3):
             try:
@@ -745,252 +773,28 @@ def test_pub_sub_default_settings(queue_and_binding):
         assert 0 == e.exit_code
 
 
-@pytest_twisted.inlineCallbacks
-def test_pub_timeout():
-    """Assert PublishTimeout is raised if a connection just hangs."""
+@pytest.fixture
+def bad_amqp_url():
+    amqp_url_backup = config.conf["amqp_url"]
     sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
     sock.bind(("", 0))
     config.conf["amqp_url"] = "amqp://localhost:{port}/".format(
         port=sock.getsockname()[1]
     )
+    yield
+    config.conf["amqp_url"] = amqp_url_backup
+    sock.close()
+    # The cached service is borked, reset it.
+    api._twisted_service = None
 
+
+@pytest_twisted.inlineCallbacks
+def test_pub_timeout(bad_amqp_url):
+    """Assert PublishTimeout is raised if a connection just hangs."""
     try:
         yield threads.deferToThread(api.publish, api.Message(), 5)
     except Exception as e:
         if not isinstance(e, exceptions.PublishTimeout):
-            pytest.fail("Expected a timeout exception, not {}".format(e))
-    finally:
-        sock.close()
+            pytest.fail(f"Expected a timeout exception, not {e}")
     # Ensure the deferred has been renewed
     assert api._twisted_service._service.factory.when_connected().called is False
-
-
-@pytest_twisted.inlineCallbacks
-def test_check_confirms():
-    """Assert confirmations are enabled by default."""
-    serv = service.FedoraMessagingService(amqp_url="amqp://")
-    serv.startService()
-    client = yield serv.getFactory().whenConnected()
-    channel = yield client._allocate_channel()
-    assert channel._delivery_confirmation is True
-    serv.stopService()
-
-
-@pytest_twisted.inlineCallbacks
-def test_basic_pub_sub():
-    """Basic test of the Twisted publishing/subscribing support"""
-    queue = str(uuid.uuid4())
-    queues = [
-        {
-            "queue": queue,
-            "auto_delete": True,
-            "durable": False,
-            "exclusive": False,
-            "arguments": {"x-expires": 60 * 1000},
-        }
-    ]
-    msg = message.Message(
-        topic=u"nice.message",
-        headers={u"niceness": u"very"},
-        body={u"encouragement": u"You're doing great!"},
-    )
-    expected_headers = {
-        u"fedora_messaging_severity": 20,
-        u"fedora_messaging_schema": u"base.message",
-        u"niceness": u"very",
-    }
-    messages_received = []
-    serv = service.FedoraMessagingService(amqp_url="amqp://")
-
-    serv.startService()
-    client = yield serv.getFactory().whenConnected()
-    yield client.declare_queues(queues)
-    yield client.bind_queues(
-        [{"queue": queue, "exchange": "amq.topic", "routing_key": "#"}]
-    )
-
-    def callback(message):
-        messages_received.append(message)
-        if len(messages_received) == 3:
-            raise exceptions.HaltConsumer()
-
-    yield client.consume(callback, queue)
-    for _ in range(0, 3):
-        yield client.publish(msg, "amq.topic")
-    yield task.deferLater(reactor, 3.0, lambda: True)
-    yield serv.stopService()
-
-    assert len(messages_received) == 3
-    for m in messages_received:
-        assert u"nice.message" == m.topic
-        assert {u"encouragement": u"You're doing great!"} == m.body
-        assert "sent-at" in m._headers
-        del m._headers["sent-at"]
-        assert expected_headers == m._headers
-
-
-@pytest_twisted.inlineCallbacks
-def test_unhandled_exception_cancels_consumer():
-    """Assert any unhandled Exception results in the consumer being canceled."""
-    queue = str(uuid.uuid4())
-    queues = [
-        {
-            "queue": queue,
-            "auto_delete": True,
-            "durable": False,
-            "exclusive": False,
-            "arguments": {"x-expires": 60 * 1000},
-        }
-    ]
-    serv = service.FedoraMessagingService(amqp_url="amqp://")
-
-    serv.startService()
-    client = yield serv.getFactory().whenConnected()
-    yield client.declare_queues(queues)
-    yield client.bind_queues(
-        [{"queue": queue, "exchange": "amq.topic", "routing_key": "#"}]
-    )
-
-    def callback(message):
-        raise Exception("Panic!")
-
-    yield client.consume(callback, queue)
-    assert len(client._consumers) == 1
-
-    yield client.publish(message.Message(), "amq.topic")
-    yield task.deferLater(reactor, 3.0, lambda: True)
-    assert len(client._consumers) == 0
-    yield serv.stopService()
-
-
-@pytest_twisted.inlineCallbacks
-def test_nack_handled():
-    """Assert raising Nack in a consumer works and messages are re-delivered"""
-    queue = str(uuid.uuid4())
-    queues = [
-        {
-            "queue": queue,
-            "auto_delete": True,
-            "durable": False,
-            "exclusive": False,
-            "arguments": {"x-expires": 60 * 1000},
-        }
-    ]
-    messages = []
-    serv = service.FedoraMessagingService(amqp_url="amqp://")
-
-    serv.startService()
-    client = yield serv.getFactory().whenConnected()
-    yield client.declare_queues(queues)
-    yield client.bind_queues(
-        [{"queue": queue, "exchange": "amq.topic", "routing_key": "#"}]
-    )
-
-    def callback(message):
-        messages.append(message)
-        if len(messages) < 3:
-            raise exceptions.Nack()
-
-    yield client.consume(callback, queue)
-    assert len(client._consumers) == 1
-
-    yield client.publish(message.Message(), "amq.topic")
-    yield task.deferLater(reactor, 3.0, lambda: True)
-
-    assert len(messages) == 3
-    assert len(set([m.id for m in messages])) == 1
-    assert len(client._consumers) == 1
-
-    yield client.cancel(queue)
-    serv.stopService()
-
-
-@pytest_twisted.inlineCallbacks
-def test_drop_handled():
-    """Assert raising Drop in a consumer works and messages are not re-delivered"""
-    queue = str(uuid.uuid4())
-    messages = []
-    serv = service.FedoraMessagingService(amqp_url="amqp://")
-    serv.startService()
-    client = yield serv.getFactory().whenConnected()
-    queues = [
-        {
-            "queue": queue,
-            "auto_delete": True,
-            "durable": False,
-            "exclusive": False,
-            "arguments": {"x-expires": 60 * 1000},
-        }
-    ]
-    yield client.declare_queues(queues)
-    yield client.bind_queues(
-        [{"queue": queue, "exchange": "amq.topic", "routing_key": "#"}]
-    )
-
-    def callback(message):
-        messages.append(message)
-        raise exceptions.Drop()
-
-    yield client.consume(callback, queue)
-    assert len(client._consumers) == 1
-
-    yield client.publish(message.Message(), "amq.topic")
-    yield task.deferLater(reactor, 3.0, lambda: True)  # Just wait a few seconds
-
-    assert len(messages) == 1
-    assert len(client._consumers) == 1
-    yield client.cancel(queue)
-    yield serv.stopService()
-
-
-@pytest_twisted.inlineCallbacks
-def test_declare_queue_failures():
-    """Assert that if a queue can't be declared, it results in an exception."""
-    serv = service.FedoraMessagingService(amqp_url="amqp://")
-    serv.startService()
-    client = yield serv.getFactory().whenConnected()
-
-    queues = [{"queue": str(uuid.uuid4()), "passive": True}]
-    try:
-        yield client.declare_queues(queues)
-    except exceptions.BadDeclaration as e:
-        assert "queue" == e.obj_type
-        assert queues[0] == e.description
-        assert isinstance(e.reason, pika.exceptions.ChannelClosed)
-    yield serv.stopService()
-
-
-@pytest_twisted.inlineCallbacks
-def test_declare_exchange_failures():
-    """Assert that if an exchange can't be declared, it results in an exception."""
-    serv = service.FedoraMessagingService(amqp_url="amqp://")
-    serv.startService()
-    client = yield serv.getFactory().whenConnected()
-
-    exchanges = [{"exchange": str(uuid.uuid4()), "passive": True}]
-    try:
-        yield client.declare_exchanges(exchanges)
-    except exceptions.BadDeclaration as e:
-        assert "exchange" == e.obj_type
-        assert exchanges[0] == e.description
-        assert isinstance(e.reason, pika.exceptions.ChannelClosed)
-    yield serv.stopService()
-
-
-@pytest_twisted.inlineCallbacks
-def test_declare_binding_failure():
-    """Assert that if a binding can't be declared, it results in an exception."""
-    serv = service.FedoraMessagingService(amqp_url="amqp://")
-    serv.startService()
-    client = yield serv.getFactory().whenConnected()
-
-    binding = [
-        {"exchange": str(uuid.uuid4()), "queue": str(uuid.uuid4()), "routing_key": "#"}
-    ]
-    try:
-        yield client.bind_queues(binding)
-    except exceptions.BadDeclaration as e:
-        assert "binding" == e.obj_type
-        assert binding[0] == e.description
-        assert isinstance(e.reason, pika.exceptions.ChannelClosed)
-    yield serv.stopService()
