@@ -21,11 +21,16 @@ from unittest import mock, TestCase
 
 import jsonschema
 import pika
+import pytest
 
 from fedora_messaging import exceptions, message
 
 
-class GetMessageTests(TestCase):
+class DeprecatedMessage(message.Message):
+    deprecated = True
+
+
+class TestGetMessage:
     """Tests for the :func:`fedora_messaging.message.get_message` function."""
 
     def test_missing_severity(self):
@@ -34,16 +39,15 @@ class GetMessageTests(TestCase):
         del msg._headers["fedora_messaging_severity"]
 
         recv_msg = message.get_message("", msg._properties, b"{}")
-        self.assertEqual(recv_msg.severity, message.INFO)
+        assert recv_msg.severity == message.INFO
 
     def test_invalid_severity(self):
         """Assert the invalid severity fails validation."""
         msg = message.Message()
         msg._headers["fedora_messaging_severity"] = 42
 
-        self.assertRaises(
-            exceptions.ValidationError, message.get_message, "", msg._properties, b"{}"
-        )
+        with pytest.raises(exceptions.ValidationError):
+            message.get_message("", msg._properties, b"{}")
 
     def test_missing_headers(self):
         """Assert missing headers results in a default message."""
@@ -53,7 +57,27 @@ class GetMessageTests(TestCase):
         received_msg = message.get_message(
             msg._encoded_routing_key, msg._properties, msg._encoded_body
         )
-        self.assertIsInstance(received_msg, message.Message)
+        assert isinstance(received_msg, message.Message)
+
+    @mock.patch.dict(
+        message._class_to_schema_name, {DeprecatedMessage: "deprecated_message_id"}
+    )
+    @mock.patch.dict(
+        message._schema_name_to_class, {"deprecated_message_id": DeprecatedMessage}
+    )
+    def test_deprecated(self, caplog):
+        """Assert a deprecation warning is produced when indicated."""
+        msg = DeprecatedMessage(topic="dummy.topic")
+        received_msg = message.get_message(
+            msg.topic, msg._properties, msg._encoded_body
+        )
+        assert isinstance(received_msg, DeprecatedMessage)
+        assert len(caplog.messages) == 1
+        assert caplog.messages[0] == (
+            "A message with a deprecated schema (fedora_messaging.tests.unit.test_message."
+            "DeprecatedMessage) has been received on topic 'dummy.topic'. You should check "
+            "the emitting application's documentation to upgrade to the newer schema version."
+        )
 
 
 class MessageDumpsTests(TestCase):
@@ -75,6 +99,7 @@ class MessageDumpsTests(TestCase):
             delivery_mode=2,
             headers=test_headers,
             message_id=test_id,
+            priority=2,
         )
         test_msg = message.Message(
             body=test_body, topic=test_topic, properties=test_properties
@@ -83,8 +108,8 @@ class MessageDumpsTests(TestCase):
         test_msg.queue = test_queue
         expected_json = (
             '{"body": {"test_key": "test_value"}, "headers": {"fedora_messaging_schema": '
-            '"base.message", "fedora_messaging_severity": 30}, "id": "test id", "queue": '
-            '"test queue", "topic": "test topic"}\n'
+            '"base.message", "fedora_messaging_severity": 30}, "id": "test id", '
+            '"priority": 2, "queue": "test queue", "topic": "test topic"}\n'
         )
         self.assertEqual(expected_json, message.dumps(test_msg))
 
@@ -115,11 +140,11 @@ class MessageDumpsTests(TestCase):
         test_msg2.queue = test_queue
         expected_json = (
             '{"body": {"test_key": "test_value"}, "headers": {"fedora_messaging_schema": '
-            '"base.message", "fedora_messaging_severity": 30}, "id": "test id", "queue": '
-            '"test queue", "topic": "test topic"}\n'
+            '"base.message", "fedora_messaging_severity": 30}, "id": "test id", '
+            '"priority": 0, "queue": "test queue", "topic": "test topic"}\n'
             '{"body": {"test_key": "test_value"}, "headers": {"fedora_messaging_schema": '
-            '"base.message", "fedora_messaging_severity": 30}, "id": "test id", "queue": '
-            '"test queue", "topic": "test topic"}\n'
+            '"base.message", "fedora_messaging_severity": 30}, "id": "test id", '
+            '"priority": 0, "queue": "test queue", "topic": "test topic"}\n'
         )
 
         self.assertEqual(expected_json, message.dumps([test_msg, test_msg2]))
@@ -138,7 +163,7 @@ class MessageLoadsTests(TestCase):
         message_json = (
             '{"topic": "test topic", "headers": {"fedora_messaging_schema": "base.message", '
             '"fedora_messaging_severity": 30}, "id": "test id", "body": '
-            '{"test_key": "test_value"}, "queue": "test queue"}\n'
+            '{"test_key": "test_value"}, "priority": 2, "queue": "test queue"}\n'
         )
         messages = message.loads(message_json)
         test_message = messages[0]
@@ -148,6 +173,7 @@ class MessageLoadsTests(TestCase):
         self.assertEqual("test id", test_message.id)
         self.assertEqual({"test_key": "test_value"}, test_message.body)
         self.assertEqual("test queue", test_message.queue)
+        self.assertEqual(2, test_message.priority)
         self.assertEqual(
             message.WARNING, test_message._headers["fedora_messaging_severity"]
         )
@@ -223,6 +249,17 @@ class MessageLoadsTests(TestCase):
         )
         self.assertRaises(exceptions.ValidationError, message.loads, message_json)
 
+    def test_missing_priority(self):
+        """Assert message without priority is accepted and the priority is set to zero."""
+        message_json = (
+            '{"topic": "test topic", "headers": {"fedora_messaging_schema": '
+            '"base.message", "fedora_messaging_severity": 30}, "id": "test id", '
+            '"body": {"test_key": "test_value"}, "queue": "test queue"}'
+        )
+        messages = message.loads(message_json)
+        test_message = messages[0]
+        self.assertEqual(test_message.priority, 0)
+
     def test_validation_failure(self):
         """Assert proper exception is raised when message validation failed."""
         message_json = (
@@ -335,6 +372,15 @@ class MessageTests(TestCase):
 
         self.assertEqual("1970-01-01T00:00:00+00:00", msg._headers["sent-at"])
 
+    def test_priority(self):
+        """Assert is set correctly."""
+        msg = message.Message()
+        self.assertEqual(0, msg.priority)
+        msg.priority = 42
+        self.assertEqual(42, msg.priority)
+        msg.priority = None
+        self.assertEqual(0, msg.priority)
+
     def test_properties(self):
         properties = object()
         msg = message.Message(properties=properties)
